@@ -1,4 +1,4 @@
-# Copyright 2011-2021 Louis Paternault
+# Copyright 2011-2022 Louis Paternault
 #
 # This file is part of pdfimpose.
 #
@@ -20,21 +20,20 @@
 import logging
 import sys
 
+from ... import UserError, pdf
+from .. import ArgumentParser
+from ..cards.__main__ import format2signature
 from . import __doc__ as DESCRIPTION
 from . import impose
-from .. import common as schema
-from ... import UserError
-from ... import pdf
-from ..cards.__main__ import format2signature
 
 
 def main(argv=None):
     """Main function"""
 
-    parser = schema.ArgumentParser(
+    parser = ArgumentParser(
         subcommand="wire",
         description=DESCRIPTION,
-        options=["omargin", "imargin", "mark", "last", "signature", "format"],
+        options=["omargin", "imargin", "mark", "last", "cutsignature", "format"],
     )
 
     try:
